<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00010000FaturamentoApuracaoFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Retorna os tipos de medida variáveis da proposta que gerou o contrato.
     */
    public function getMedidasVariaveisDaPropostaDoContrato(array $params) {
        $sql = "SELECT REF.IDCONTRATIVO,
                       REF.NRSEQORCAMENTO,
                       REF.NRSEQCONTRATO,
                       REL.IDELEMENTORAIZ,
                       ELE.NMSERVORCA AS NMELEMENTORAIZ,
                       REL.NRRELACMEDIDAPROPOSTA,
                       REL.CDTPMEDIDAVARIAVEL,
                       MEDIDA.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                       REL.QTMEDIDAVARIAVEL
                  FROM FCT_RELACMEDIDAPROPOSTA REL
                 INNER JOIN FCT_REFEORCACONT REF
                    ON REL.NRSEQORCAMENTO = REF.NRSEQORCAMENTO
                   AND REL.NRORG          = REF.NRORG
                 INNER JOIN FCT_TIPOMEDIDASERV MEDIDA
                    ON REL.CDTPMEDIDAVARIAVEL = MEDIDA.CDTPMEDIDASERV
                   AND REL.NRORG              = MEDIDA.NRORG
                  LEFT JOIN FCT_SERVICOORCA ELE
                    ON REL.IDELEMENTORAIZ = ELE.IDELEMENTO
                   AND REL.NRORG          = ELE.NRORG
                 WHERE REF.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND REF.NRORG         = :NRORG
                 ORDER BY REF.IDCONTRATIVO, REF.NRSEQORCAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna a proposta que gerou o contrato.
     */
    public function getPropostaDoContrato(array $params) {
        $sql = "SELECT ORC.NRSEQORCAMENTO,
                       ORC.NRORCAMENTOMODELO,
                       NVL(REF.NRTIPOORCAMENTO, ORC.NRTIPOORCAMENTO) AS NRTIPOORCAMENTO
                  FROM FCT_REFEORCACONT REF
                 INNER JOIN FCT_ORCAMENTO ORC
                    ON REF.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                   AND REF.NRORG          = ORC.NRORG
                 WHERE REF.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND REF.NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
     /**
     * Retorna os CPF das pessoas do contrato.
     */
    public function getCPFpessoas(array $params) {
        $sql = "SELECT DISTINCT GPE_PESSOAH.NRCPFPESSOA, FCT_RECURSOSAGENDCONT.NRPARCNEPESSOA
                    FROM GPE_PESSOAH
                    JOIN GPE_PESSOA 
                      ON GPE_PESSOA.NRPESSOA = GPE_PESSOAH.NRPESSOA
                     AND GPE_PESSOA.NRORG = GPE_PESSOAH.NRORG
                    JOIN FCT_RECURSOSAGENDCONT
                      ON FCT_RECURSOSAGENDCONT.NRPARCNEPESSOA = GPE_PESSOA.NRPARCNEGOCIO
                     AND FCT_RECURSOSAGENDCONT.NRORG = GPE_PESSOA.NRORG
                     AND ((:DTINICOMPET BETWEEN FCT_RECURSOSAGENDCONT.DTINIAGENDRECURSO AND FCT_RECURSOSAGENDCONT.DTFIMAGENDRECURSO) OR 
                          (:DTFINCOMPET BETWEEN FCT_RECURSOSAGENDCONT.DTINIAGENDRECURSO AND FCT_RECURSOSAGENDCONT.DTFIMAGENDRECURSO))
                    JOIN FCT_AGENDRECURSOCONT 
                     ON  FCT_AGENDRECURSOCONT.NRAGENDRECURSOCONT = FCT_RECURSOSAGENDCONT.NRAGENDRECURSOCONT
                     AND FCT_AGENDRECURSOCONT.NRORG = FCT_RECURSOSAGENDCONT.NRORG
                    JOIN FCT_VENCPREVCONT 
                      ON  FCT_VENCPREVCONT.NRSEQCONTRATO = FCT_AGENDRECURSOCONT.NRSEQCONTRATO
                      AND FCT_VENCPREVCONT.NRORG = FCT_AGENDRECURSOCONT.NRORG
                      AND FCT_VENCPREVCONT.NRVENCPREVCONT = :NRVENCPREVCONT
                    JOIN FCT_ITVENCPREVCONT 
                      ON FCT_ITVENCPREVCONT.NRVENCPREVCONT =  FCT_VENCPREVCONT.NRVENCPREVCONT
                     AND FCT_ITVENCPREVCONT.NRORG = FCT_VENCPREVCONT.NRORG
                     AND FCT_ITVENCPREVCONT.NRITEMOCONTRATO = FCT_RECURSOSAGENDCONT.NRITEMOCONTRATO
                     WHERE GPE_PESSOAH.NRORG = :NRORG
                     AND FCT_RECURSOSAGENDCONT.IDSITUARECGEND = '1'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
       /**
     * Retorna o codigo intem do contrato.
     */
    public function getItenDoContrato(array $params) {
        //die;
        $sql = "SELECT NRITEMOCONTRATO FROM FCT_ITVENCPREVCONT
                        WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                        AND NRSEQCONTRATO = :NRSEQCONTRATO 
                        AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
     
     /**
     * Retorna a Url de integração com o Hcm
    **/
    public function getUrlIntegracaoHcm(array $params){
        $sql = "SELECT URLINTEGRACAOHCM 
                FROM FCT_PARAMETROGERAL
                WHERE NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();        
    }
    
    /**
     * Retorna os itens do vencimento
     */
    public function getItensFaturamento(array $params) {
        $sql = "SELECT ITVENC.NRVENCPREVCONT,
                       ITVENC.NRITVENCPREVCONT,
                       ITVENC.NRSEQCONTRATO,
                       ITVENC.NRITEMOCONTRATO,
                       ITVENC.VRVENCIMENTO,
                       ITCONT.CDSERVORCA,
                       ITVENC.NRORG
                   FROM FCT_ITVENCPREVCONT ITVENC
                  INNER JOIN FCT_ITEMCONTRATO ITCONT
                     ON ITVENC.NRSEQCONTRATO   = ITCONT.NRSEQCONTRATO
                    AND ITVENC.NRITEMOCONTRATO = ITCONT.NRITEMOCONTRATO
                    AND ITVENC.NRORG           = ITCONT.NRORG
                  WHERE ITVENC.NRVENCPREVCONT = :NRVENCPREVCONT
                    AND ITVENC.NRSEQCONTRATO  = :NRSEQCONTRATO
                    AND ITVENC.NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Atualiza o valor do item do vencimento
     */
    public function atualizarItemVencimento(array $params) {
        $sql = "UPDATE FCT_ITVENCPREVCONT
                   SET VRVENCIMENTO = :VRVENCIMENTO
                 WHERE NRVENCPREVCONT   = :NRVENCPREVCONT
                   AND NRITVENCPREVCONT = :NRITVENCPREVCONT
                   AND NRSEQCONTRATO    = :NRSEQCONTRATO
                   AND NRORG            = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Atualiza o valor do item do vencimento
     */
    public function atualizarItemVencimentoHoras(array $params) {
        $sql = "UPDATE FCT_ITVENCPREVCONT
                   SET VRVENCIMENTO = VRVENCIMENTO + :VRVENCIMENTO
                 WHERE NRVENCPREVCONT   = :NRVENCPREVCONT
                   AND NRSEQCONTRATO    = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO  = :NRITEMOCONTRATO
                   AND NRORG            = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Atualiza o valor total do vencimento
     */
    public function atualizarValorTotalVencimento(array $params) {
        $sql = "UPDATE FCT_VENCPREVCONT
                   SET VRVENCIMENTO   = :VALOR,
                       VRFALTAFATURAR = :VALOR
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna os itens do vencimento com as datas de início e fim do contrato e da competência atual
     */
    public function getParametrosVencimento(array $params) {
        $sql = "SELECT CONT.IDTPFATUINCOMP,
                       TO_CHAR(CONT.DTINICIOCONTRATO, 'DD/MM/YYYY') AS DTINICIOCONTRATO,
                       TO_CHAR(CONT.DTFIMCONTRATO, 'DD/MM/YYYY') AS DTFIMCONTRATO,
                       TO_CHAR(VENC.DTINICOMPET, 'DD/MM/YYYY') AS DTINICOMPET,
                       TO_CHAR(VENC.DTFINCOMPET, 'DD/MM/YYYY') AS DTFINCOMPET
                  FROM FCT_VENCPREVCONT VENC
                 INNER JOIN FCT_CONTRATO CONT
                    ON VENC.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                   AND VENC.NRORG         = CONT.NRORG
                 WHERE VENC.NRVENCPREVCONT = :NRVENCPREVCONT
                   AND VENC.NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND VENC.NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Verifica se o contrato usa as horas apontadas nos agendamentos no faturamento
     */
    public function getIdApuraApontamentoContrato(array $params) {
        $sql = "SELECT IDAPURAAPONTAMENTO
                  FROM FCT_PARAMFATCONT
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG         = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getTotalHorasApontadasContrato(array $params) {
        $sql = "SELECT NVL(SUM(HORASSOMADAS), 0) AS HORAS
                  FROM (
                    SELECT APROPRIACAO.*,
                           HORASAPROPRIADAS - TEMPOBASE AS HORASADICIONAIS,
                           DECODE(IDHORAEXTRA, 'N', HORASAPROPRIADAS,
                              DECODE(VRPERCHORAEXTRA, NULL, HORASAPROPRIADAS,
                                  (CASE
                                     WHEN HORASFERIADO > QTHORASINI THEN
                                          (HORASAPROPRIADAS + (HORASFERIADO * VRPERCHORAEXTRA / 100))
                                     WHEN HORASAPROPRIADAS - TEMPOBASE > QTHORASINI THEN
                                          (HORASAPROPRIADAS + ((HORASAPROPRIADAS - TEMPOBASE - QTHORASINI) * VRPERCHORAEXTRA / 100))
                                     ELSE HORASAPROPRIADAS END)
                              )
                           ) AS HORASSOMADAS
                      FROM (
                            SELECT APONTAMENTO.*,
                                   DECODE(IDDIAUTIL, 'S', QTHORASAPROPRIADAS, DECODE(IDDESCANSOREMU, 'N', QTHORASAPROPRIADAS, TEMPOBASE)) AS HORASAPROPRIADAS,
                                   DECODE(IDDIAUTIL, 'S', 0, DECODE(IDDESCANSOREMU, 'N', 0, QTHORASAPROPRIADAS)) AS HORASFERIADO
                              FROM (
                                    SELECT APRO.NRAPROPAGENDAMENTO,
                                           APRO.DTAPROPRIACAO,
                                           APRO.IDSTATUSAPROPRIACAO,
                                           REC.NRPARCNEPESSOA,
                                           PARC.NMPRINCIPALPARC,
                                           REC.IDHORAEXTRA,
                                           REC.IDDESCANSOREMU,
                                           APRO.QTHORASAPROPRIADAS,
                                           APRO.IDDIAUTIL,
                                           EXTRA.VRPERCHORAEXTRA,
                                           HORACHAR_HORADEC(EXTRA.QTHORASINI) AS QTHORASINI,
                                           (HORACHAR_HORADEC(REC.HRPRIMEIRASAIDA) - HORACHAR_HORADEC(REC.HRPRIMEIRAENTRADA)) + (HORACHAR_HORADEC(REC.HRSEGUNDASAIDA) - HORACHAR_HORADEC(REC.HRSEGUNDAENTRADA)) AS TEMPOBASE
                                      FROM FCT_APROPAGENDAMENTO APRO
                                     INNER JOIN FCT_RECURSOSAGENDCONT REC
                                        ON APRO.NRRECURSOSAGENDCONT = REC.NRRECURSOSAGENDCONT
                                       AND APRO.NRORG               = REC.NRORG
                                     INNER JOIN PARCNEGOCIO PARC
                                        ON REC.NRPARCNEPESSOA = PARC.NRPARCNEGOCIO
                                       AND REC.NRORG          = PARC.NRORG
                                     INNER JOIN FCT_AGENDRECURSOCONT AGEN
                                        ON REC.NRAGENDRECURSOCONT = AGEN.NRAGENDRECURSOCONT
                                       AND REC.NRORG              = AGEN.NRORG
                                      LEFT JOIN FCT_AGENDRECHORAEXTRA EXTRA
                                        ON AGEN.NRAGENDRECURSOCONT = EXTRA.NRAGENDRECURSOCONT
                                       AND AGEN.NRORG              = EXTRA.NRORG
                                       AND APRO.IDDIASEMANA        = EXTRA.IDDIASEMANA
                                     WHERE AGEN.NRSEQCONTRATO = :NRSEQCONTRATO
                                       AND AGEN.NRORG         = :NRORG
                                       AND TRUNC(APRO.DTAPROPRIACAO) BETWEEN :DATAINI AND :DATAFIN
                                ) APONTAMENTO
                        ) APROPRIACAO
                     WHERE APROPRIACAO.IDSTATUSAPROPRIACAO = '3'
                     OR (IDDESCANSOREMU = 'S' AND IDDIAUTIL = 'N')
                     ORDER BY APROPRIACAO.DTAPROPRIACAO)";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getTotalHorasAgendadasContrato(array $params) {
        $sql = "SELECT SUM(TEMPOBASE) AS HORAS
                  FROM (
                    SELECT APRO.NRAPROPAGENDAMENTO,
                           APRO.DTAPROPRIACAO,
                           REC.NRPARCNEPESSOA,
                           REC.IDHORAEXTRA,
                           REC.IDDESCANSOREMU,
                           APRO.QTHORASAPROPRIADAS,
                           APRO.IDDIAUTIL,
                           (HORACHAR_HORADEC(REC.HRPRIMEIRASAIDA) - HORACHAR_HORADEC(REC.HRPRIMEIRAENTRADA)) + (HORACHAR_HORADEC(REC.HRSEGUNDASAIDA) - HORACHAR_HORADEC(REC.HRSEGUNDAENTRADA)) AS TEMPOBASE
                      FROM FCT_APROPAGENDAMENTO APRO
                     INNER JOIN FCT_RECURSOSAGENDCONT REC
                        ON APRO.NRRECURSOSAGENDCONT = REC.NRRECURSOSAGENDCONT
                       AND APRO.NRORG               = REC.NRORG
                     INNER JOIN FCT_AGENDRECURSOCONT AGEN
                        ON REC.NRAGENDRECURSOCONT = AGEN.NRAGENDRECURSOCONT
                       AND REC.NRORG              = AGEN.NRORG
                     WHERE AGEN.NRSEQCONTRATO = :NRSEQCONTRATO
                       AND AGEN.NRORG         = :NRORG
                       AND TRUNC(APRO.DTAPROPRIACAO) BETWEEN :DATAINI AND :DATAFIN
                     ORDER BY APRO.DTAPROPRIACAO)";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getValorVencimento(array $params) {
        $sql = "SELECT VRVENCIMENTO
                  FROM FCT_VENCPREVCONT
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRORG          = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getValorTotalItensVencimento(array $params) {
        $sql = "SELECT SUM(VRVENCIMENTO) AS VRVENCIMENTO
                  FROM FCT_ITVENCPREVCONT
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRORG          = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Retorna os Itens da Contrato.
     */
    public function getItensContrato(array $params) {
        $sql = "SELECT ITEM.NRITEMOCONTRATO,
                       ITEM.NMITEMCONTRATO,
                       ITEM.CDSERVORCA,
                       REFE.NRTIPOORCAMENTO,
                       ITEM.QTITEMCONTRATO,
                       ITEM.VRUNITITEMCONT,
                       ITEM.VRTOTITEMCONT,
                       ITEM.IDTPDESCONTO,
                       ITEM.VRDESCONTO,
                       ITEM.QTMEDIDACONTR
                  FROM FCT_ITEMCONTRATO ITEM
                  INNER JOIN FCT_REFEORCACONT REFE
                    ON ITEM.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                   AND ITEM.NRORG         = REFE.NRORG
                 WHERE ITEM.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND ITEM.NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os Itens da Contrato.
     */
    public function getItensContratoHoras(array $params) {
        $sql = "SELECT ITEM.NRITEMOCONTRATO,
                       ITEM.NMITEMCONTRATO,
                       ITEM.CDSERVORCA,
                       REFE.NRTIPOORCAMENTO,
                       ITEM.QTITEMCONTRATO,
                       ITEM.VRUNITITEMCONT,
                       ITEM.VRTOTITEMCONT,
                       ITEM.IDTPDESCONTO,
                       ITEM.VRDESCONTO,
                       ITEM.QTMEDIDACONTR
                  FROM FCT_ITEMCONTRATO ITEM
                  LEFT JOIN FCT_REFEORCACONT REFE
                    ON ITEM.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                   AND ITEM.NRORG         = REFE.NRORG
                 WHERE ITEM.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND ITEM.NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna os Itens da Contrato.
     */
    public function getEventos(array $params) {
        $sql = "SELECT *
                  FROM FCT_TMP_INTEGRACAOHCM
                 WHERE NRVENCPREVCONT      = :NRVENCPREVCONT
                   AND NRSEQCONTRATO       = :NRSEQCONTRATO
                   AND NRPARCNEGOCIOPESSOA = :NRPARCNEGOCIOPESSOA
                   AND NRORG               = :NRORG
                   AND (INSTR('_'||:NRSEVENTOS||'_', '_'||NREVENTO||'_') > 0)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
     /**
     * Retorna os tempo base de carga horária do contrato.
     */
    public function buscaAgendamentosDeRecurso(array $params) {
        $sql = "SELECT NRPARCNEPESSOA,
                       NRITEMOCONTRATO,
                       SUM(TEMPOBASE) AS QTHORAS,
                       COUNT(*) AS QTDIAS
                  FROM (
                    SELECT REC.NRPARCNEPESSOA,
                           REC.NRITEMOCONTRATO,
                           TO_CHAR(APRO.DTAPROPRIACAO, 'DD/MM/YYYY') AS DTAPROPRIACAO,
                           (HORACHAR_HORADEC(REC.HRPRIMEIRASAIDA) - HORACHAR_HORADEC(REC.HRPRIMEIRAENTRADA)) + (HORACHAR_HORADEC(REC.HRSEGUNDASAIDA) - HORACHAR_HORADEC(REC.HRSEGUNDAENTRADA)) AS TEMPOBASE
                     FROM FCT_APROPAGENDAMENTO APRO
                    INNER JOIN FCT_RECURSOSAGENDCONT REC
                       ON APRO.NRRECURSOSAGENDCONT = REC.NRRECURSOSAGENDCONT
                      AND APRO.NRORG               = REC.NRORG
                      AND REC.IDSITUARECGEND       = 1
                    INNER JOIN FCT_AGENDRECURSOCONT AGEN
                       ON REC.NRAGENDRECURSOCONT = AGEN.NRAGENDRECURSOCONT
                      AND REC.NRORG              = AGEN.NRORG
                    WHERE AGEN.NRSEQCONTRATO = :NRSEQCONTRATO
                      AND AGEN.NRORG         = :NRORG
                      AND TRUNC(APRO.DTAPROPRIACAO) BETWEEN :DATAINI AND :DATAFIN
                    ORDER BY REC.NRPARCNEPESSOA, REC.NRITEMOCONTRATO, APRO.DTAPROPRIACAO
                       ) AGENDAMENTOS
                 GROUP BY NRPARCNEPESSOA, NRITEMOCONTRATO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Salva dados na tabela temporaria
     */
    public function saveTabelaTemporaria(array $params){
        
        $sql= "INSERT INTO FCT_TMP_INTEGRACAOHCM (
                    NRSEQCONTRATO,
                    NRITEMOCONTRATO,
                    NRPARCNEGOCIOPESSOA,
                    NRVENCPREVCONT,
                    DTREFERENCIA,
                    IDTIPOEVENTO,
                    IDUNIDREFEREVEN,
                    VRREFFOLHA,
                    NREVENTO,
                    NRORG
                )
                VALUES (
                    :NRSEQCONTRATO,
                    :NRITEMOCONTRATO,
                    :NRPARCNEGOCIOPESSOA,
                    :NRVENCPREVCONT,
                    :DTREFERENCIA,
                    :IDTIPOEVENTO,
                    :IDUNIDREFEREVEN,
                    :VRREFFOLHA,
                    :NREVENTOM,
                    :NRORG 
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
    /**
     * Deleta os dados na tabela temporaria
     */
    public function deleteTabelaTemporaria(array $params){
        
        $sql= "DELETE FROM FCT_TMP_INTEGRACAOHCM
                  WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                    AND NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
    /**
     * Valida os dados na tabela temporaria
     */
    public function validaTabelaTemporaria(array $params) {
        $sql= "SELECT COUNT(*) AS VALIDA
                 FROM FCT_TMP_INTEGRACAOHCM
                WHERE NRSEQCONTRATO       = :NRSEQCONTRATO
                  AND NRVENCPREVCONT      = :NRVENCPREVCONT
                  AND NRPARCNEGOCIOPESSOA = :NRPARCNEGOCIOPESSOA
                  AND NRORG               = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /*public function getTotalHorasAgendadasContratoHora(array $params) {
        $sql = "SELECT APRO.NRAPROPAGENDAMENTO,
                       APRO.DTAPROPRIACAO,
                       REC.NRPARCNEPESSOA,
                       REC.IDHORAEXTRA,
                       REC.IDDESCANSOREMU,
                       APRO.QTHORASAPROPRIADAS,
                       APRO.IDDIAUTIL,
                       (HORACHAR_HORADEC(REC.HRPRIMEIRASAIDA) - HORACHAR_HORADEC(REC.HRPRIMEIRAENTRADA)) + (HORACHAR_HORADEC(REC.HRSEGUNDASAIDA) - HORACHAR_HORADEC(REC.HRSEGUNDAENTRADA)) AS TEMPOBASE
                  FROM FCT_APROPAGENDAMENTO APRO
                 INNER JOIN FCT_RECURSOSAGENDCONT REC
                    ON APRO.NRRECURSOSAGENDCONT = REC.NRRECURSOSAGENDCONT
                   AND APRO.NRORG               = REC.NRORG
                 INNER JOIN FCT_AGENDRECURSOCONT AGEN
                    ON REC.NRAGENDRECURSOCONT = AGEN.NRAGENDRECURSOCONT
                   AND REC.NRORG              = AGEN.NRORG
                 WHERE AGEN.NRSEQCONTRATO        = :NRSEQCONTRATO
                 AND REC.NRITEMOCONTRATO         = :NRITEMOCONTRATO
                   AND AGEN.NRORG                = :NRORG
                   AND TRUNC(APRO.DTAPROPRIACAO) = :DTAPROPRIACAO
                   AND REC.NRPARCNEPESSOA        = :NRPARCNEPESSOA
                 ORDER BY APRO.DTAPROPRIACAO";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }*/
    
    /**
     * limpa o valor do item do vencimento
     */
    public function limparVrItemVencimentoHoras(array $params) {
        $sql = "UPDATE FCT_ITVENCPREVCONT
                   SET VRVENCIMENTO = 0
                 WHERE NRVENCPREVCONT   = :NRVENCPREVCONT
                   AND NRSEQCONTRATO    = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO  = :NRITEMOCONTRATO
                   AND NRORG            = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function salvarTabelaRelatorioMedidaFaturamento(array $params) {
        $sql = "INSERT INTO FCT_RELATMEDFATURA (
                    NRSEQCONTRATO    , NRITEMOCONTRATO , NRVENCPREVCONT     , NRPARCNEGPESSOA ,
					NRHORASAGENDADAS , NRHORASDIA      , NRHORASTRABALHADAS , VRTRABALHADO    ,
					QTHREXTRA50      , VRHREXTRA50     , QTHREXTRA70        , VRHREXTRA70     ,
					QTHREXTRA100     , VRHREXTRA100    , QTTOTHREXTRAS      , VRTOTHREXTRAS   ,
					VRTOTMEDICAO     , NRORG
                )
                VALUES (
                    :NRSEQCONTRATO    , :NRITEMOCONTRATO , :NRVENCPREVCONT     , :NRPARCNEGPESSOA ,
					:NRHORASAGENDADAS , :NRHORASDIA      , :NRHORASTRABALHADAS , :VRTRABALHADO    ,
					:QTHREXTRA50      , :VRHREXTRA50     , :QTHREXTRA70        , :VRHREXTRA70     ,
					:QTHREXTRA100     , :VRHREXTRA100    , :QTTOTHREXTRAS      , :VRTOTHREXTRAS   ,
					:VRTOTMEDICAO     , :NRORG
                )";
                
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getDadosItemVencimento(array $params) {
        $sql = "SELECT VRUNITITEMCONT, QTITEMCONTRATO, VRTOTITEMCONT, VRVENCIMENTO
                  FROM FCT_ITVENCPREVCONT
                 WHERE NRSEQCONTRATO   = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                   AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function excluirTabelaRelatorioMedicaoFaturamento(array $params) {
        $sql = "DELETE FROM FCT_RELATMEDFATURA
                 WHERE NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRORG          = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Atualiza o valor de todos is itens do vencimento
     */
    public function atualizarValorTodosItensVencimento(array $params) {
        $sql = "UPDATE FCT_ITVENCPREVCONT
                   SET VRVENCIMENTO = :VRVENCIMENTO
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getNomePesssoa(array $params) {
        $sql = "SELECT NMPRINCIPALPARC
                  FROM PARCNEGOCIO
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function recebeValorItens(array $params) {
        $sql = "SELECT SUM(VRTOTITEMCONT) AS VRITENS
                  FROM FCT_ITVENCPREVCONT
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function atualizarValorParcela(array $params) {
        $sql = "UPDATE FCT_VENCPREVCONT
                   SET VRVENCIMENTO = :VRITENS,
                       VRFALTAFATURAR = :VRITENS,
                       IDALTUSUARIO = 'S',
                       NRORGULTATU = :NRORG,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :CDOPERADOR
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function criaParcelaNotaAnte(array $params) {
        $sql = "INSERT INTO FCT_VENCPREVCONT (NRVENCPREVCONT, NRSEQCONTRATO, NRGRUPOFATURA, IDFATURACONTR, CDMODPGTO, NRAPURACAO, QTDIASVENCIMENTO, DTINICOMPET, 
                                              DTFINCOMPET, DTVENCIMENTO, VRVENCIMENTO, DTEMISSAO, IDALTUSUARIO, IDSITUVENCPREV, NRAGRUPFATUR, NRPARCNEGOCIO, 
                                              VRFATURADO, VRFALTAFATURAR, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, IDATIVO)
                                      VALUES (:NRVENCPREVCONT, :NRSEQCONTRATO, :NRGRUPOFATURA, :IDFATURACONTR, :CDMODPGTO, :NRAPURACAO, :QTDIASVENCIMENTO, :DTINICOMPET, 
                                              :DTFINCOMPET, :DTVENCIMENTO, :VRVENCIMENTO, :DTEMISSAO, :IDALTUSUARIO, :IDSITUVENCPREV, :NRAGRUPFATUR, :NRPARCNEGOCIO, 
                                              :VRFATURADO, :VRFALTAFATURAR, :NRORG, :NRORG, SYSDATE, :OPERADOR, 'S')";
                
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function recebeDadosItens(array $params) {
        $sql = "SELECT NRITVENCPREVCONT, NRVENCPREVCONT, NRSEQCONTRATO, NRITEMOCONTRATO, IDFATURACONTR, CDMODPGTO, QTDIASVENCIMENTO, DTINICOMPET, DTFINCOMPET,
                       DTVENCIMENTO, DTEMISSAO, VRVENCIMENTO, QTITEMCONTRATO, CDTPMEDIDASERV, QTMEDIDACONTR, VRUNITITEMCONT, VRTOTITEMCONT, IDALTUSUARIO,
                       NRORG, QTPARCELASFATU, NRDIAVENCPARCELASFATU, IDSITUITVENCPREV
                  FROM FCT_ITVENCPREVCONT
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT 
                   AND NRSEQCONTRATO = :NRSEQCONTRATO 
                   AND NRORG = :NRORG
                   AND IDATIVO = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function criaItensParcelaNotaAnte(array $params) {
        $sql = "INSERT INTO FCT_ITVENCPREVCONT (NRITVENCPREVCONT, NRVENCPREVCONT, NRSEQCONTRATO, NRITEMOCONTRATO, IDFATURACONTR, CDMODPGTO, QTDIASVENCIMENTO, DTINICOMPET, DTFINCOMPET, 
                                                DTVENCIMENTO, DTEMISSAO, VRVENCIMENTO, QTITEMCONTRATO, CDTPMEDIDASERV, QTMEDIDACONTR, VRUNITITEMCONT, VRTOTITEMCONT, IDALTUSUARIO, 
                                                QTPARCELASFATU, NRDIAVENCPARCELASFATU, IDSITUITVENCPREV, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, IDATIVO)
                                        VALUES (:NRITVENCPREVCONT, :NRVENCPREVCONT, :NRSEQCONTRATO, :NRITEMOCONTRATO, :IDFATURACONTR, :CDMODPGTO, :QTDIASVENCIMENTO, :DTINICOMPET, :DTFINCOMPET,
                                                :DTVENCIMENTO, :DTEMISSAO, :VRVENCIMENTO, :QTITEMCONTRATO, :CDTPMEDIDASERV, :QTMEDIDACONTR, :VRUNITITEMCONT, :VRTOTITEMCONT, :IDALTUSUARIO,
                                                :QTPARCELASFATU, :NRDIAVENCPARCELASFATU, :IDSITUITVENCPREV, :NRORG, :NRORG, SYSDATE, :OPERADOR, 'S')";
                                                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function atualizaValorItemParcelaNotaAnte(array $params) {
        $sql = "UPDATE FCT_ITVENCPREVCONT
                   SET VRVENCIMENTO = :VRVENCIMENTO
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT 
                   AND NRSEQCONTRATO = :NRSEQCONTRATO 
                   AND NRORG = :NRORG";
                                                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function atualizaValorParcelaNotaAnte(array $params) {
        $sql = "UPDATE FCT_VENCPREVCONT
                   SET VRVENCIMENTO = :VALOR, 
                       VRFATURADO = 0, 
                       VRFALTAFATURAR = :VALOR
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT 
                   AND NRSEQCONTRATO = :NRSEQCONTRATO 
                   AND NRORG = :NRORG";
                                                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function atualizaParcelaOriNotaAnte(array $params) {
        $sql = "UPDATE FCT_VENCPREVCONT
                   SET NRVENCPREVCONTANTECIP = :NRVENCPREVCONTANTECIP,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG = :NRORG";
                                                                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /*----------------------------------------
     | --- Nova rotina de  Faturamento   --- |
     ----------------------------------------*/
     
    public function anularApuracao(array $params) {
        $sql = "UPDATE FCT_FATURACONT
                SET DSJUSTIFICATIVA = :DSJUSTIFICATIVA,
                    IDSITUACAO      = '4',
                    VRFATURADO      = 0,
                    VRFALTAFATURAR  = VRVENCIMENTO,
                    DTULTATU        = SYSDATE,
                    NRORGULTATU     = :NRORG,
                    CDOPERULTATU    = :CDOPERADOR
              WHERE NRFATURACONT    = :NRFATURACONT
                AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function criaApuracaoManual(array $params) {
        $sql = "INSERT INTO FCT_APURACAO
                        (NRAPURACAO, NRSEQCONTRATO, NRTIPOORCAMENTO,
                         NRPARCNEGOCIO, CDMODPGTO, NRGRUPOFATURA,
                         CDEMPRESA, CDFILIAL, NRLANCTONF,
                         IDSITUACAOAPURACAO, VRAPURACAO, VRAPURACAOORIGINAL,
                         DSALTERACAO, CDAGRUPANF, NRORG,
                         DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                  VALUES
                        (:NRAPURACAO, :NRSEQCONTRATO, NULL,
                         :NRPARCFATURA, NULL, :NRGRUPOFATURA,
                         :CDEMPRESA, :CDFILIAL, :NRLANCTONF,
                         'FATURADA', :VRNOTAFISC, :VRVENCIMENTO,
                         NULL, NULL, :NRORG,
                         SYSDATE, :NRORG, :CDOPERADOR)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function atualizaParcelaApuracaoManual(array $params) {
        $sql = "UPDATE FCT_FATURACONT
                SET NRAPURACAO      = :NRAPURACAO,
                    DSJUSTIFICATIVA = :DSJUSTIFICATIVA,
                    IDSITUACAO      = '3',
                    VRFATURADO      = :VRNOTAFISC,
                    VRFALTAFATURAR  = 0,
                    IDFORMAFATURA   = 'M',
                    DTULTATU        = SYSDATE,
                    NRORGULTATU     = :NRORG,
                    CDOPERULTATU    = :CDOPERADOR
              WHERE NRFATURACONT  = :NRFATURACONT
                AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function getDadosParcela(array $params) {
        $sql = "SELECT NRFATURACONT, NRSEQCONTRATO, NRGRUPOFATURA, CDFILIALFATURA, NRPARCFATURA, IDTPFATURA, 
                    IDSITUACAO, DTINICOMPET, DTFINCOMPET, DTLIMITEAPU, DTVENCIMENTO, NRDIASVENCIMENTO, 
                    QTDIASFATUVENC, VRVENCIMENTO, VRFATURADO, VRFALTAFATURAR, IDFORMAFATURA, NRAPURACAO, 
                    NRMEDICAOFATURA, DSJUSTIFICATIVA, IDATIVO, NRORG, CDTIPOOPERFATURA, CDSERIENF,
                    CDEMPRESA, CDINSCESTA, CDINSCMUNI
                FROM FCT_FATURACONT
                WHERE NRFATURACONT = :NRFATURACONT
                    AND NRORG      = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function insereParcelaDesmembramento(array $params) {
        $sql = "INSERT INTO FCT_FATURACONT
                    (NRFATURACONT, NRSEQCONTRATO, NRGRUPOFATURA, CDFILIALFATURA, NRPARCFATURA, IDTPFATURA, 
                     IDSITUACAO, DTINICOMPET, DTFINCOMPET, DTLIMITEAPU, DTVENCIMENTO, NRDIASVENCIMENTO, 
                     QTDIASFATUVENC, VRVENCIMENTO, VRFATURADO, VRFALTAFATURAR, IDFORMAFATURA, DSJUSTIFICATIVA, 
                     NRAPURACAO, NRMEDICAOFATURA, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO,
                     CDTIPOOPERFATURA, CDSERIENF, CDEMPRESA, CDINSCESTA, CDINSCMUNI)
                VALUES 
                    (:NRFATURACONT, :NRSEQCONTRATO, :NRGRUPOFATURA, :CDFILIALFATURA, :NRPARCFATURA, :IDTPFATURA, 
                     :IDSITUACAO, :DTINICOMPET, :DTFINCOMPET, :DTLIMITEAPU, :DTVENCIMENTO, :NRDIASVENCIMENTO, 
                     :QTDIASFATUVENC, :VRVENCIMENTO, :VRFATURADO, :VRFALTAFATURAR, :IDFORMAFATURA, :DSJUSTIFICATIVA, 
                     :NRAPURACAO, :NRMEDICAOFATURA, 'S', :NRORG, SYSDATE, :NRORG, :OPERADOR, :CDTIPOOPERFATURA, :CDSERIENF,
                     :CDEMPRESA, :CDINSCESTA, :CDINSCMUNI)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function getDadosItem(array $params) {
        $sql = "SELECT NRITFATURACONT, NRFATURACONT, NRSEQCONTRATO, NRITEMOCONTRATO, CDPRODFATURA, 
                       IDSITUACAO, VRVENCIMENTO, VRFATURADO, VRFALTAFATURAR, IDFORMAFATURA, 
                       NRAPURACAO, NRORG
                  FROM FCT_ITFATURACONT
                 WHERE NRITFATURACONT = :NRITFATURACONT
                   AND NRFATURACONT   = :NRFATURACONT
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function insereItemDesmembramento(array $params) {
        $sql = "INSERT INTO FCT_ITFATURACONT 
                   (NRITFATURACONT, NRFATURACONT, NRSEQCONTRATO, NRITEMOCONTRATO, CDPRODFATURA, 
                    IDSITUACAO, VRVENCIMENTO, VRFATURADO, VRFALTAFATURAR, IDFORMAFATURA, 
                    DSJUSTIFICATIVA, NRAPURACAO, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                VALUES
                   (:NRITFATURACONT, :NRFATURACONT, :NRSEQCONTRATO, :NRITEMOCONTRATO, :CDPRODFATURA, 
                    :IDSITUACAO, :VRVENCIMENTO, :VRFATURADO, :VRFALTAFATURAR, :IDFORMAFATURA, 
                    :DSJUSTIFICATIVA, :NRAPURACAO, 'S', :NRORG, SYSDATE, :NRORG, :OPERADOR)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function atualizaValorParcelaNova(array $params) {
        $sql = "UPDATE FCT_FATURACONT
                   SET VRVENCIMENTO   = VRVENCIMENTO + :VRVENCIMENTO,
                       VRFALTAFATURAR = VRFALTAFATURAR + :VRVENCIMENTO
                 WHERE NRFATURACONT = :NRFATURACONT
                   AND NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function DeletaItemParcelaOrigem(array $params) {
        $sql = "DELETE FROM FCT_ITFATURACONT
                 WHERE (NRITFATURACONT = :NRITFATURACONT
                        OR :NRITFATURACONT IS NULL)
                   AND NRFATURACONT = :NRFATURACONT
                   AND (NRSEQCONTRATO = :NRSEQCONTRATO
                        OR :NRSEQCONTRATO IS NULL)
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function atualizaValorParcelaOrigem(array $params) {
        $sql = "UPDATE FCT_FATURACONT
                   SET VRVENCIMENTO   = VRVENCIMENTO - :VRVENCIMENTO,
                       VRFALTAFATURAR = VRFALTAFATURAR - :VRVENCIMENTO,
                       DSJUSTIFICATIVA = :DSJUSTIFICATIVA
                 WHERE NRFATURACONT = :NRFATURACONT
                   AND NRORG        = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function validaQtItens(array $params) {
        $sql = "SELECT COUNT(*) AS VALIDA 
                  FROM FCT_ITFATURACONT
                 WHERE NRFATURACONT  = :NRFATURACONT
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getJustificativa(array $params) {
        $sql = "SELECT DSJUSTIFICATIVA
                FROM FCT_FATURACONT
                WHERE NRFATURACONT  = :NRFATURACONT
                  AND NRORG         = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Faz o select das informaçãoes das parcelas tentando agrupa-las
    **/
    public function validarParcelasParaAgrupamento($nrfaturacont, $nrorg){
        $sql = "SELECT NRGRUPOFATURA,
                    CDFILIALFATURA,
                    NRPARCFATURA,
                    IDTPFATURA,
                    TRUNC(DTINICOMPET) AS DTINICOMPET,
                    TRUNC(DTFINCOMPET) AS DTFINCOMPET,
                    SUM(VRVENCIMENTO) AS VRVENCIMENTO,
                    MIN(DTLIMITEAPU) AS DTLIMITEAPU,
                    MIN(DTVENCIMENTO) AS DTVENCIMENTO,
                    1 AS IDSITUACAO,
                    NULL AS NRDIASVENCIMENTO,
                    NULL AS QTDIASFATUVENC,
                    NULL AS IDFORMAFATURA,
                    NULL AS NRAPURACAO,
                    NULL AS NRMEDICAOFATURA,
                    CDTIPOOPERFATURA, 
                    CDSERIENF,
                    CDEMPRESA, 
                    CDINSCESTA, 
                    CDINSCMUNI
                FROM (SELECT NRGRUPOFATURA, CDFILIALFATURA, NRPARCFATURA,
                        IDTPFATURA, TRUNC(DTINICOMPET) AS DTINICOMPET,
                        TRUNC(DTFINCOMPET) AS DTFINCOMPET,
                        VRVENCIMENTO, DTLIMITEAPU,
                        DTVENCIMENTO, CDTIPOOPERFATURA, CDSERIENF,
                        CDEMPRESA, CDINSCESTA, CDINSCMUNI
                      FROM FCT_FATURACONT
                      WHERE NRFATURACONT IN($nrfaturacont)
                          AND NRORG      = $nrorg
                          AND IDSITUACAO = 1)
                GROUP BY NRGRUPOFATURA, CDFILIALFATURA,
                  NRPARCFATURA, IDTPFATURA,
                  DTINICOMPET, DTFINCOMPET, 1,
                  CDTIPOOPERFATURA, CDSERIENF,
                  CDEMPRESA, CDINSCESTA, 
                  CDINSCMUNI";
        $ps = $this->connection->prepare($sql);
        $ps->execute();
        return $ps->fetchAll();
    }
   
    /**
     * Cria um novo relacionamento da parcela com o contrato
    **/
    public function salvarRelacAgrupContFat($params){
        $sql = "INSERT INTO FCT_RELACAGRUPCONTFAT(
                    NRAGRUPCONTFAT,   IDPARCAGRUP,  NRSEQCONTRATO,  NRORG, 
                    DTINCLUSAO,       DTULTATU,     NRORGINCLUSAO,  CDOPERINCLUSAO, 
                    NRORGULTATU,      CDOPERULTATU, NRFATURACONT,   NRMEDICAOFATURA,
                    IDATIVO
                ) VALUES (
                    :NRAGRUPCONTFAT,  :IDPARCAGRUP,  :NRSEQCONTRATO,  :NRORG, 
                    SYSDATE,          NULL,          :NRORG,          :OPERADOR, 
                    :NRORG,           :OPERADOR,     :NRFATURACONT,   :NRMEDICAOFATURA,
                    'S'
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);    
    }
   
    /**
     * Deleta o relacionamento de parcelas na tabela fct_relaagrupcontfat
    **/
    public function deletarRelacAgrupamentoContFat($nrfaturacont, array $params){
        $sql = "DELETE FROM FCT_RELACAGRUPCONTFAT
                WHERE NRFATURACONT IN($nrfaturacont)
                    AND NRORG = :NRORG
                    AND (NRSEQCONTRATO = :NRSEQCONTRATO 
                            OR :NRSEQCONTRATO IS NULL)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);   
    }
   
    /**
     * Deleta as parcelas 
    **/
    public function deletarParcelaFatura($nrfaturacont, $nrorg){
        $sql = "DELETE FROM FCT_FATURACONT
                WHERE NRFATURACONT IN($nrfaturacont)
                    AND NRORG = $nrorg";
        $std = $this->connection->prepare($sql);
        $std->execute();   
    }
    
    /**
     * Retorna as informações dos relacionamentos da parcela com o contrato e com a 
     * medição
    **/
    public function getInformacoesRelacParcelaCont($nrfaturacont, array $params){
        $sql = "SELECT NRSEQCONTRATO,
                    NRMEDICAOFATURA
                FROM FCT_RELACAGRUPCONTFAT
                WHERE NRFATURACONT IN($nrfaturacont)
                    AND NRORG = :NRORG
                    AND (NRSEQCONTRATO = :NRSEQCONTRATO OR :NRSEQCONTRATO IS NULL)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
        return $std->fetchAll();
    }
    
    /**
     * Retorna todos os itens de um contrato determinado em uma parcela
    **/
    public function getItemsFatura($params, $nrfaturacont){
        $sql = "SELECT NRITFATURACONT, NRFATURACONT, NRSEQCONTRATO, NRITEMOCONTRATO, CDPRODFATURA, 
                       IDSITUACAO, VRVENCIMENTO, VRFATURADO, VRFALTAFATURAR, IDFORMAFATURA, 
                       NRAPURACAO, NRORG
                  FROM FCT_ITFATURACONT
                 WHERE NRFATURACONT  IN ($nrfaturacont)
                   AND (NRSEQCONTRATO = :NRSEQCONTRATO
                        OR :NRSEQCONTRATO IS NULL)
                   AND NRORG         = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
        return $std->fetchAll();
    }
    /**
     * Caso a parcela esteja ligada a apenas 1 contrato no final do agrupamento
     * muda o status para parcela normal
    **/
    public function alterarInfoAgrupamentoParcela($params){
        $sql = "UPDATE FCT_RELACAGRUPCONTFAT
                SET IDPARCAGRUP  = 'N',
                    CDOPERULTATU = :CDOPERADOR
                WHERE NRFATURACONT  = :NRFATURACONT
                   AND NRORG        = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
    }
    /**
     * Insere um relacionamento entre a apuração e a parcela
    **/
    public function inserirRelacApuracaoFatura(array $params){
        $sql = "INSERT INTO FCT_RELACAPURACAOFATURA (
                    NRAPURACAOFATURA, NRAPURACAO,    NRFATURACONT,
                    NRORG,            IDATIVO,       DTINCLUSAO,
                    DTULTATU,         NRORGINCLUSAO, CDOPERINCLUSAO,
                    NRORGULTATU,      CDOPERULTATU
                ) VALUES (
                    :NRAPURACAOFATURA, :NRAPURACAO,  :NRFATURACONT,
                    :NRORG,            'S',           SYSDATE,
                    NULL,              :NRORG,       :CDOPERADOR,
                    NULL,              NULL
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
    }
    /**
     * Retorna todas as parcelas ligadas a um agrupamento
    **/
    public function getParcelasAgrupamentoCont(array $params, $nrseqcontrato){
        $sql = "SELECT DISTINCT AGRUPFAT.NRFATURACONT
                FROM FCT_RELACAGRUPCONTFAT AGRUPFAT
                INNER JOIN FCT_RELACAGRUPAMENTOCONT AGRUPCONT  
                    ON AGRUPFAT.NRSEQCONTRATO = AGRUPCONT.NRSEQCONTRATO
                    AND AGRUPFAT.NRORG        = AGRUPCONT.NRORG
                INNER JOIN FCT_FATURACONT FAT
                    ON AGRUPFAT.NRFATURACONT = FAT.NRFATURACONT
                    AND AGRUPFAT.NRORG       = FAT.NRORG
                WHERE AGRUPCONT.NRSEQCONTRATO IN($nrseqcontrato)
                    AND AGRUPCONT.NRAGRUPAMENTO = :NRAGRUPAMENTO
                    AND AGRUPFAT.NRORG          = :NRORG
                    AND IDSITUACAO              = '1'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
        return $std->fetchAll();
    }
    /**
     * Deleta o contrato do agrupamento
    **/
    public function deletarContratoAgrupamento($nrseqcontrato, $params){
        $sql = "DELETE FROM FCT_RELACAGRUPAMENTOCONT
                WHERE NRSEQCONTRATO IN($nrseqcontrato)
                    AND NRAGRUPAMENTO = :NRAGRUPAMENTO
                    AND NRORG         = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
    }
    
    /**
     * Deleta o agrupamento
    **/
    public function deletarAgrupamento($params){
        $sql = "DELETE FROM FCT_AGRUPAMENTO
                WHERE NRAGRUPAMENTO = :NRAGRUPAMENTO
                    AND NRORG         = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
    }
    
    /**
     * Deleta o agrupamento
    **/
    public function deletarEscoposAgrupamento($params){
        $sql = "DELETE FROM FCT_RELACESCOPOAGRUPAMENTO
                WHERE NRAGRUPAMENTO = :NRAGRUPAMENTO
                    AND NRORG         = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
    }
    
    /**
     * Retorna um array com os numeros de contratos de um determinado agrupamento
    **/
    public function getContratosAgrupamento(array $params){
        $sql = "SELECT LISTAGG(NRSEQCONTRATO, ',') WITHIN GROUP (ORDER BY NRSEQCONTRATO) AS NRSEQCONTRATO
                FROM FCT_RELACAGRUPAMENTOCONT
                WHERE NRAGRUPAMENTO = :NRAGRUPAMENTO
                    AND NRORG       = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
        $dataset = $std->fetchAll();
        
        if(isset($dataset) && count($dataset) > 0 && $dataset[0]["NRSEQCONTRATO"] != "")
            return explode(",", $dataset[0]["NRSEQCONTRATO"]);
        return null;
    }
    /**
     * Valida se há dados na tabela do relatório
    **/
    public function getPossuiRelatorioEspelho(array $params){
        $sql = "SELECT COUNT(*) AS POSSUIRELAT
                FROM FCT_RELATMEDFATURA
                WHERE NRSEQCONTRATO   = :NRSEQCONTRATO
                   AND NRVENCPREVCONT = :NRVENCPREVCONT
                   AND NRORG          = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    
    public function validarAgrupamentoFat(array $params){
        $sql = "SELECT CONT.NRCONTRATOFCT,
                     RELCONT.NRSEQCONTRATO,
                     RELCONT.NRORG
                FROM FCT_RELACAGRUPAMENTOCONT RELCONT
                JOIN FCT_CONTRATO CONT
                  ON RELCONT.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                  AND RELCONT.NRORG        = CONT.NRORG
                WHERE RELCONT.NRAGRUPAMENTO IN (
                      SELECT DISTINCT NRAGRUPAMENTO
                      FROM FCT_RELACAGRUPAMENTOCONT
                      WHERE (INSTR(:NRSEQCONTRATO, '_' || TO_CHAR(NRSEQCONTRATO) || '_') > 0)
                    )
                  AND RELCONT.NRSEQCONTRATO NOT IN (
                      SELECT NRSEQCONTRATO
                      FROM FCT_RELACAGRUPCONTFAT
                      WHERE NRFATURACONT = :NRFATURACONT
                    )
                  AND RELCONT.NRSEQCONTRATO IN (
                      SELECT NRSEQCONTRATO
                      FROM FCT_CONTRATO
                      WHERE IDSITUACAOCONTRATO = 'P'
                      UNION ALL 
                      SELECT NRSEQCONTRATO
                      FROM FCT_MEDICAOFATURA
                      WHERE NRPARCFATURA        = :NRPARCFATURA
                        AND TRUNC(DTINICOMPET) >= :DTINICIO 
                        AND TRUNC(DTINICOMPET) <= :DTFIM
                        AND ((IDSITUACAO = '3' AND VRMEDICAO <> 0)
                              OR IDSITUACAO IN ('2', '1'))        
                    )
                  AND RELCONT.NRORG = :NRORG";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Valida se existem contratos para o período que não tiveram reajuste.
    **/
    public function validarReajuste($params) {
        $sql = "SELECT FCT_CONTRATO.NRSEQCONTRATO,
                       FCT_CONTRATO.NRCONTRATOFCT, 
                       FCT_CONTRATO.NRPARCNEGOCIO,
                       PARCNEGOCIO.NMPRINCIPALPARC,  
                       NVL(FCT_CONTRATO.DTINIVIGENCONT, FCT_CONTRATO.DTINICIOCONTRATO) AS DTINICIO, 
                       NVL(FCT_CONTRATO.DTFINVIGENCONT, FCT_CONTRATO.DTFIMCONTRATO) AS DTFINAL,
                       FCT_CONTRATO.DSREFERENCIACONT
                  FROM FCT_CONTRATO
                 INNER JOIN PARCNEGOCIO
                    ON FCT_CONTRATO.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                   AND FCT_CONTRATO.NRORG         = PARCNEGOCIO.NRORG
                  LEFT JOIN FCT_REFEORCACONT
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCT_REFEORCACONT.NRSEQCONTRATO
                   AND FCT_CONTRATO.NRORG         = FCT_REFEORCACONT.NRORG
                 WHERE FCT_CONTRATO.IDSITUACAOCONTRATO = 'A'
                   AND FCT_CONTRATO.NRSEQCONTRATO IN (SELECT DISTINCT NEG.NRSEQCONTRATO FROM FCT_NEGOCIACAOCONTRATO NEG WHERE NEG.IDTPFATURA IN ('A', 'X'))
                   AND TRUNC(NVL(FCT_CONTRATO.DTFINVIGENCONT, FCT_CONTRATO.DTFIMCONTRATO)) <= TRUNC(SYSDATE)
                   AND FCT_CONTRATO.NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);  
        return $std->fetchAll();
    }
    
    public function deletaTempContPendenciaReaj(array $params){
        
        $sql = "DELETE FROM FCT_TMP_CONTRPENDENCIAREAJ
                 WHERE NRORG = :NRORG";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function insereTempContPendenciaReaj(array $params){
        
        $sql = "INSERT INTO FCT_TMP_CONTRPENDENCIAREAJ (NRSEQCONTRATO, NRCONTRATOFCT, NRPARCNEGOCIO, NMPRINCIPALPARC,
                                        DTINICIOCONTRATO, DTFIMCONTRATO, DSREFERENCIACONT, NRORG,
                                        DTINCLUSAO)
                                VALUES (:NRSEQCONTRATO, :NRCONTRATOFCT, :NRPARCNEGOCIO, :NMPRINCIPALPARC,
                                        :DTINICIOCONTRATO, :DTFIMCONTRATO, :DSREFERENCIACONT, :NRORG,
                                        SYSDATE)";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Altera a situação da parcela para faturado
    **/
    public function alterarSituacaoParcela(array $params) {
        $sql = "UPDATE FCT_FATURACONT
                SET IDSITUACAO   = '3',
                    DTULTATU     = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU  = :NRORG 
                WHERE NRFATURACONT = :NRFATURACONT
                    AND NRORG      = :NRORG";
                       
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Altera a situação dos itens da parcela para faturado
    **/
    public function alterarSituacaoItens(array $params) {
        $sql = "UPDATE FCT_ITFATURACONT
                SET IDSITUACAO   = '3',
                    DTULTATU     = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU  = :NRORG 
                WHERE NRFATURACONT = :NRFATURACONT
                    AND NRORG      = :NRORG";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Altera a situação dos rateios da parcela para faturado
    **/
    public function alterarSituacaoRateio(array $params) {
        $sql = "UPDATE FCT_RATEIOITFATURACONT
                SET IDSITUVENCRATEIO = '3',
                    DTULTATU         = SYSDATE,
                    CDOPERULTATU     = :CDOPERULTATU,
                    NRORGULTATU      = :NRORG 
                WHERE NRFATURACONT = :NRFATURACONT
                    AND NRORG      = :NRORG";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    

    public function validaReajusteFatu(array $params) {
        $sql = "SELECT IDBLOQAPURFATSEMREAJ
                  FROM FCT_PARAMETROGERAL
                 WHERE NRORG  = :NRORG";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params); 
        return $ps->fetchAll();
    }
    
    /**
     * Altera a situação e a data de saída da nota fiscal
    **/
    public function alterarDataSaidaDaNotaFiscal(array $params) {
        $sql = "UPDATE NOTAFISCAL
                SET IDSTATUS     = :IDSTATUS,
                    DTENTRSAID   = :DTENTRSAID,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU  = :NRORG
                WHERE CDFILIAL   = :CDFILIAL  
                  AND NRLANCTONF = :NRLANCTONF
                  AND NRORG      = :NRORG";
                  
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
}

?>