<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00004109NegociacaoContratoFactory;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\ConnectionCustom;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Session\Session;

class Sm00004109NegociacaoContratoService {
    protected $negociacaofactory;
    protected $entityManager;
    protected $connectioncustom;
    protected $connection;
    
    public function __construct(Sm00004109NegociacaoContratoFactory $negociacaofactory, EntityManager $entityManager, ConnectionCustom $connectioncustom, NewCode $newCode) {
        $this->negociacaofactory = $negociacaofactory;
        $this->entityManager     = $entityManager;
        $this->connectioncustom  = $connectioncustom;
        $this->connection        = $entityManager->getConnection();
        $this->newCode           = $newCode;
    }
    
    
    
    /**
     * Insere os dados de Negociação de Contrato
    **/
    public function saveDadosNegContrato($row, $nrorg, $cdoperador) {
        
        $this->connection->beginTransaction();
        
        $negociacao = $row["NEGOCIACAO"];
        $parcelas = $row["PARCELAS"];
        
        try { 
            
            $nrnegcontrato = $this->newCode->getCodeByTableName("FCT_NEGCONTRATO", $nrorg, 1, 10, true);
            
            $params = array(
                "NRNEGCONTRATO"     => $nrnegcontrato,
                "NRPARCNEGOCIO"     => $negociacao["NRPARCNEGOCIO"], 
                "NRAGRUPAMENTO"     => $negociacao["NRAGRUPAMENTO"], 
                "IDTPFATURA"        => $negociacao["IDTPFATURA"], 
                "CDFILIAL"          => $negociacao["CDFILIAL"], 
                "NRPARCFATURA"      => $negociacao["NRPARCFATURA"], 
                "CDTIPOOPERFATURA"  => $negociacao["CDTIPOOPERFATURA"], 
                "CDSERIENF"         => $negociacao["CDSERIENF"], 
                "CDEMPRESA"         => $negociacao["CDEMPRESA"], 
                "CDINSCESTA"        => $negociacao["CDINSCESTA"], 
                "CDINSCMUNI"        => $negociacao["CDINSCMUNI"], 
                "NRSEQCONTRATO"     => $negociacao["NRSEQCONTRATO"], 
                "NRITEMOCONTRATO"   => $negociacao["NRITEMOCONTRATO"], 
                "NRITEMELEMENTO"    => $negociacao["NRITEMELEMENTO"], 
                "DSVARIAVELNEGOCIA" => $negociacao["DSVARIAVELNEGOCIA"], 
                "IDTIPONEGOCIA"     => $negociacao["IDTIPONEGOCIA"], 
                "DSOBSNEGOCIA"      => $negociacao["DSOBSNEGOCIA"], 
                "IDATIVO"           => 'S',
                "NRORG"             => $nrorg,
                "OPERADOR"          => $cdoperador
            );
            
            $this->negociacaofactory->saveDadosNegContrato($params);
            
            /** adiciona as parcelas de Negociação **/
            
            foreach($parcelas as $parcela){
            
                $dtiniparcnegocia = DateUtil::getDataDeString($parcela["DTINIPARCNEGOCIA"]);
                $dtfimparcnegocia = DateUtil::getDataDeString($parcela["DTFIMPARCNEGOCIA"]);
                
                $datainicial = DateUtil::truncateData($dtiniparcnegocia);
                $datafinal   = DateUtil::truncateData($dtfimparcnegocia);
                
                if ($parcela["IDMULTIPLICAQTMED"] == 'Y'){
                    $idmultiplicaqtmed = 'S';
                }
                else {
                   $idmultiplicaqtmed = 'N'; 
                }
                
                $valor = $parcela["VRPARCELANEGOCIA"];
                
                $this->saveParcelasNegContrato($nrnegcontrato, $datainicial, $datafinal, $idmultiplicaqtmed, $valor, $nrorg, $cdoperador, 'S');
            }
            
            $this->connection->commit();
        
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Gera as parcelas a serem inseridas.
    **/
    public function gerarParcelasNegContrato($nrnegcontrato, $datainicial, $datafinal, $idmultiplicaqtmed, $valor, $nrorg, $cdoperador, $substituir = 'S') {
        
        $dataini = $datainicial;
        $datafim = DateUtil::getUltimoDiaDoMes($datainicial);
    
        /* Se o utltimo dia do mês da data inicial for maior que a data final informada pelo usuário,
           é inserida uma parcela com as datas que o usuário informou, se não são geradas as datas
           mês a mês*/    
        if ($datafim >= $datafinal) {
            
            $this->saveParcelasNegContrato($nrnegcontrato, $datainicial, $datafinal, $idmultiplicaqtmed, $valor, $nrorg, $cdoperador, $substituir);
        }
        else{
            /*Enquanto a data informada pelo usuário for maior que o ultimo dia do mês das datas iniciais calculadas, 
              é inserida uma nova parcela com as datas calculadas */
            while($datafinal >= $datafim){
                
                $this->saveParcelasNegContrato($nrnegcontrato, $dataini, $datafim, $idmultiplicaqtmed, $valor, $nrorg, $cdoperador, $substituir);
                
                $dataini = DateUtil::adicionaIntervalo($datafim,1);
                $datafim = DateUtil::getUltimoDiaDoMes($dataini);
               
                /*quando a data final calculada for maior ou igual a data informada pelo usuário, 
                  é inserida a última parcela com a data final informada pelo usuário, */
                if ($datafim >= $datafinal) {
                    
                    $this->saveParcelasNegContrato($nrnegcontrato, $dataini, $datafinal, $idmultiplicaqtmed, $valor, $nrorg, $cdoperador, $substituir);
                }
            }
        }
    }
    
    /**
     * Insere as parcelas de Negociação
    **/
    public function saveParcelasNegContrato($nrnegcontrato, $datainicial, $datafinal, $idmultiplicaqtmed, $valor, $nrorg, $cdoperador, $substituir) {
        try { 
            
            if ($substituir == 'S'){
               
                $params = array(
                    "NRNEGCONTRATO"      => $nrnegcontrato,
                    "DTINIPARCNEGOCIA"   => $datainicial->format("d/m/Y"), 
                    "DTFIMPARCNEGOCIA"   => $datafinal->format("d/m/Y"), 
                    "NRORG"              => $nrorg
                ); 
                
                $this->negociacaofactory->deleteParcelasNegContrato($params);
                
                $inserir = 'S';
                
            }
            else {
                
                $params = array(
                    "NRNEGCONTRATO"      => $nrnegcontrato,
                    "DTINIPARCNEGOCIA"   => $datainicial->format("d/m/Y"), 
                    "DTFIMPARCNEGOCIA"   => $datafinal->format("d/m/Y"), 
                    "NRORG"              => $nrorg
                ); 
                
                $valida = $this->negociacaofactory->validaParcelasPeriodo($params);
                
                if ($valida == 0) {
                    
                    $inserir = 'S';
                }
                else {
                    
                    $inserir = 'N';
                }
            }
            
            if ($inserir == 'S') {
                
                $nrparcelasnegcontr = $this->newCode->getCodeByTableName("FCT_PARCELASNEGCONTR", $nrorg, 1, 10, true);
                
                $params = array(
                    "NRPARCELASNEGCONTR" => $nrparcelasnegcontr,
                    "NRNEGCONTRATO"      => $nrnegcontrato,
                    "DTINIPARCNEGOCIA"   => $datainicial->format("d/m/Y"), 
                    "DTFIMPARCNEGOCIA"   => $datafinal->format("d/m/Y"), 
                    "IDMULTIPLICAQTMED"  => $idmultiplicaqtmed, 
                    "VRPARCELANEGOCIA"   => $valor, 
                    "IDATIVO"            => 'S',
                    "NRORG"              => $nrorg,
                    "OPERADOR"           => $cdoperador
                );
                
                $this->negociacaofactory->saveParcelasNegContrato($params);
            }
        
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    /**
     * Editar os dados das parcelas de Negociação de Contrato
    **/
    public function editParcelasNegoContr($row, $nrorg, $cdoperador) {
        
        $existeParcelaSendoUtilizada = false;
        
        $this->connection->beginTransaction();
        try {
            foreach ($row as $dados) {
                $parcelaUtilizada = $this->verificarUsoParcela($dados["NRPARCELASNEGCONTR"], $nrorg);
                if ($parcelaUtilizada === false) {

                    if ($dados["IDMULTIPLICAQTMED"] == 'Y') {
                        $idMultiplicaQtMed = 'S';
                    }
                    else {
                        $idMultiplicaQtMed = 'N';
                    }
                    
                    $params = array(
                        "DTINIPARCNEGOCIA"   => $dados["DTINIPARCNEGOCIA"],
                        "DTFIMPARCNEGOCIA"   => $dados["DTFIMPARCNEGOCIA"],
                        "IDMULTIPLICAQTMED"  => $idMultiplicaQtMed,
                        "VRPARCELANEGOCIA"   => $dados["VRPARCELANEGOCIA"],
                        "NRORG"              => $nrorg,
                        "OPERADOR"           => $cdoperador,
                        "NRPARCELASNEGCONTR" => $dados["NRPARCELASNEGCONTR"],
                        "NRNEGCONTRATO"      => $dados["NRNEGCONTRATO"]
                    );
                    
                    $this->negociacaofactory->editParcelasNegoContr($params);
                }
                else {
                    $existeParcelaSendoUtilizada = true;
                }
            }
            
            $this->connection->commit();
            
            return $existeParcelaSendoUtilizada;
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Adiciona parcelas de Negociação de Contrato
    **/
    public function adicionarParcelasNegContr($row, $nrorg, $cdoperador) {
        
        try { 
            
            /** gera as parcelas de Negociação **/
            
            $nrnegcontrato     = $row["NRNEGCONTRATO"];
            $dtiniparcnegocia  = DateUtil::getDataDeString($row["DTINIPARCNEGOCIA"]);
            $dtfimparcnegocia  = DateUtil::getDataDeString($row["DTFIMPARCNEGOCIA"]);
            $idmultiplicaqtmed = $row["IDMULTIPLICAQTMED"];
            $vrparcelanegocia  = $row["VRPARCELANEGOCIA"];
            $idSubstitui       = $row["IDSUBSTITUI"];
            
            $datainicial = DateUtil::truncateData($dtiniparcnegocia);
            $datafinal = DateUtil::truncateData($dtfimparcnegocia);
            
            $this->gerarParcelasNegContrato($nrnegcontrato, $datainicial, $datafinal, $idmultiplicaqtmed, $vrparcelanegocia, $nrorg, $cdoperador, $idSubstitui);
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Valida se o período que está sendo inserido já existe em outra negociação com as mesmas variáveis para o mesmo cliente
     */
    public function validarParcelaPeriodoExistenteEmOutraNegociacao($nrnegcontrato, $dtini, $dtfin, $nrorg) {
        $params = array(
            "NRNEGCONTRATO" => $nrnegcontrato,
            "NRORG"         => $nrorg
        );
        
        $dadosNegociacao = $this->negociacaofactory->getDadosNegociacao($params);
        $dadosNegociacao = $dadosNegociacao[0];
        
        $params = array(
            "DTINI"            => $dtini,
            "DTFIN"            => $dtfin,
            "NRPARCNEGOCIO"    => $dadosNegociacao["NRPARCNEGOCIO"],
            "NRAGRUPAMENTO"    => $dadosNegociacao["NRAGRUPAMENTO"],
            "CDTIPOOPERFATURA" => $dadosNegociacao["CDTIPOOPERFATURA"],
            "CDSERIENF"        => $dadosNegociacao["CDSERIENF"],
            "CDEMPRESA"        => $dadosNegociacao["CDEMPRESA"],
            "CDINSCESTA"       => $dadosNegociacao["CDINSCESTA"],
            "CDINSCMUNI"       => $dadosNegociacao["CDINSCMUNI"],
            "NRORG"            => $dadosNegociacao["NRORG"],
            "NRSEQCONTRATO"    => $dadosNegociacao["NRSEQCONTRATO"],
            "NRITEMOCONTRATO"  => $dadosNegociacao["NRITEMOCONTRATO"]
        );
        
        $dataset = $this->negociacaofactory->getNegociacaoRepetida($params);
        if (count($dataset) > 0) {
            throw new \Exception("Operação bloqueada! <br/><br/>Já existe uma parcela inserida para o período informado em outro registro de Negociação de Contrato com as mesmas variáveis.");
        }
    }
    
    public function recebeMinMaxParcelas($nrnegcontrato, $nrorg) {
        
        $params = array(
            "NRNEGCONTRATO" => $nrnegcontrato,
            "NRORG"         => $nrorg
        );
        
        $datas = $this->negociacaofactory->recebeMinMaxParcelas($params);
        
        if(!empty($datas)){
                
            $minDtParcelas = $datas[0]["DTMINPARCELAS"];
            $maxDtParcelas = $datas[0]["DTMAXPARCELAS"];
        }
        else {
            
            $minDtParcelas = null;
            $maxDtParcelas = null;
        }
        
        $retorno = array(
            "DTMINPARCELAS" => $minDtParcelas,
            "DTMAXPARCELAS" => $maxDtParcelas 
        );
        
        return $retorno;
    }
    
    /**
     *Exclui a parcela de negociação de contrato
    **/
    public function excluirParcela($nrparcelasnegcontr, $nrnegcontrato, $nrorg) {
        
        try { 
            $params = array(
                "NRPARCELASNEGCONTR" => $nrparcelasnegcontr, 
                "NRNEGCONTRATO"      => $nrnegcontrato,
                "NRORG"              => $nrorg
            );
            
            $this->negociacaofactory->excluirParcela($params);
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
     /**
     * Gera as parcelas a serem inseridas.
    **/
    public function gerarParcelasNegContratoWiz($datainicial, $datafinal, $idmultiplicaqtmed, $valor) {
        
        $datainicial = DateUtil::getDataDeString($datainicial);
        $datafinal = DateUtil::getDataDeString($datafinal);

        $datainicial = DateUtil::truncateData($datainicial);
        $datafinal = DateUtil::truncateData($datafinal);
        
        $dataini = $datainicial;
        $datafim = DateUtil::getUltimoDiaDoMes($datainicial);
        
        if ($idmultiplicaqtmed == 'S'){
            
            $idmultiplicaqtmed = 'Y';
        }
    
        /* Se o utltimo dia do mês da data inicial for maior que a data final informada pelo usuário,
           é inserida uma parcela com as datas que o usuário informou, se não são geradas as datas
           mês a mês*/    
        if ($datafim >= $datafinal) {
            
            $retorno[0] = array(
                "DTINIPARCNEGOCIA"  => $datainicial->format("d/m/Y"),
                "DTFIMPARCNEGOCIA"  => $datafinal->format("d/m/Y"), 
                "IDMULTIPLICAQTMED" => $idmultiplicaqtmed,
                "VRPARCELANEGOCIA"  => $valor
            );
            
        }
        else{
            $count = 0;
            /*Enquanto a data informada pelo usuário for maior que o ultimo dia do mês das datas iniciais calculadas, 
              é inserida uma nova parcela com as datas calculadas */
            while($datafinal >= $datafim){
                
                $retorno[] = array(
                    "DTINIPARCNEGOCIA"  => $dataini->format("d/m/Y"),
                    "DTFIMPARCNEGOCIA"  => $datafim->format("d/m/Y"), 
                    "IDMULTIPLICAQTMED" => $idmultiplicaqtmed,
                    "VRPARCELANEGOCIA"  => $valor
                );
                
                $dataini = DateUtil::adicionaIntervalo($datafim,1);
                $datafim = DateUtil::getUltimoDiaDoMes($dataini);
               
                /*quando a data final calculada for maior ou igual a data informada pelo usuário, 
                  é inserida a última parcela com a data final informada pelo usuário, */
                if ($datafim >= $datafinal) {
                    
                    $retorno[] = array(
                        "DTINIPARCNEGOCIA"  => $dataini->format("d/m/Y"),
                        "DTFIMPARCNEGOCIA"  => $datafinal->format("d/m/Y"), 
                        "IDMULTIPLICAQTMED" => $idmultiplicaqtmed,
                        "VRPARCELANEGOCIA"  => $valor
                    );
                }
            }
        }
        
        return $retorno;
    }
    
    /**
     * Valida se ja existe um registro de negociação com os mesmos dados da que esta sendo inserida.
    **/
    public function validarNegociacoesInseridas($row, $nrorg) {
         
        $nrparcnegocio   = $row["NRPARCNEGOCIO"];
        $nragrupamento   = $row["NRAGRUPAMENTO"];
        $nrseqcontrato   = $row["NRSEQCONTRATO"];
        $nritemocontrato = $row["NRITEMOCONTRATO"];
        $nritemelemento  = $row["NRITEMELEMENTO"]; 
        
        $dtinicial       = $row["DTINICIO"];
        $dtfinal         = $row["DTFINAL"];
        
        $dtinicial = DateUtil::getDataDeString($dtinicial);
        $dtfinal = DateUtil::getDataDeString($dtfinal);

        $dtinicial = DateUtil::truncateData($dtinicial);
        $dtfinal = DateUtil::truncateData($dtfinal);
        
        $valida = 'OK';
         
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $negociacoes = $this->negociacaofactory->NegociacoesParceiro($params);
        
        foreach($negociacoes as $negociacao){
            
            if (!empty($nragrupamento)){
                
                if (($nragrupamento == $negociacao["NRAGRUPAMENTO"]) && ($nrseqcontrato == $negociacao["NRSEQCONTRATO"]) && ($nritemocontrato == $negociacao["NRITEMOCONTRATO"]) && ($nritemelemento == $negociacao["NRITEMELEMENTO"])){
                    
                    $valida = $this->validarPeriodoNegociacoesInseridas($negociacao["NRNEGCONTRATO"], $dtinicial->format("d/m/Y"), $dtfinal->format("d/m/Y"), $nrorg);
                    
                }
                else if (($nragrupamento == $negociacao["NRAGRUPAMENTO"]) && ($nrseqcontrato == $negociacao["NRSEQCONTRATO"]) && ($nritemocontrato == $negociacao["NRITEMOCONTRATO"])) {
                   
                    $valida = $this->validarPeriodoNegociacoesInseridas($negociacao["NRNEGCONTRATO"], $dtinicial->format("d/m/Y"), $dtfinal->format("d/m/Y"), $nrorg); 
                }
                else if (($nragrupamento == $negociacao["NRAGRUPAMENTO"]) && (empty($nrseqcontrato))){
                    
                    $valida = $this->validarPeriodoNegociacoesInseridas($negociacao["NRNEGCONTRATO"], $dtinicial->format("d/m/Y"), $dtfinal->format("d/m/Y"), $nrorg); 
                }
            }    
            else {    
                if (($nrseqcontrato == $negociacao["NRSEQCONTRATO"]) && ($nritemocontrato == $negociacao["NRITEMOCONTRATO"]) && ($nritemelemento == $negociacao["NRITEMELEMENTO"])){
                    
                    $valida = $this->validarPeriodoNegociacoesInseridas($negociacao["NRNEGCONTRATO"], $dtinicial->format("d/m/Y"), $dtfinal->format("d/m/Y"), $nrorg);
                      
                }
                else if (($nrseqcontrato == $negociacao["NRSEQCONTRATO"]) && ($nritemocontrato == $negociacao["NRITEMOCONTRATO"])) {
                   
                    $valida = $this->validarPeriodoNegociacoesInseridas($negociacao["NRNEGCONTRATO"], $dtinicial->format("d/m/Y"), $dtfinal->format("d/m/Y"), $nrorg); 
                }
            }
        }
        
        return $valida;
    }
    
    public function validarPeriodoNegociacoesInseridas($nrnegcontrato, $dtinicial, $dtfinal, $nrorg) {
        
        $params = array(
            "NRNEGCONTRATO" => $nrnegcontrato,
            "NRORG"         => $nrorg
        );
        
        $datas = $this->negociacaofactory->recebeMinMaxParcelasNeg($params);
        
        $valida = 'OK'; 
        if(!empty($datas)){
            
            $minDtParcelas = $datas[0]["DTMINPARCELAS"];
            $maxDtParcelas = $datas[0]["DTMAXPARCELAS"];

            
            $datainicial = DateUtil::getDataDeString($dtinicial);
            $datafinal   = DateUtil::getDataDeString($dtfinal);
            
            $dataMin = DateUtil::getDataDeString($minDtParcelas);
            $dataMax = DateUtil::getDataDeString($maxDtParcelas);
            
            
            if (($datainicial <= $dataMin) && ($datafinal >= $dataMax)){
                
                 $valida = 'NAO';
                
            }
            else if (($datainicial >= $dataMin) && ($datainicial <= $dataMax)) {
                
                $valida = 'NAO';
                
            }
            else if (($datafinal <= $dataMax) && ($datafinal >= $dataMin)) {
                 
                $valida = 'NAO';
                
            }
        }
        
        return $valida;
    }
    
    /**
     * Verifica se a parcela da negociação está sendo utilizada em alguma medição ou apuração de faturamento.
     * Retorna true ou false.
     */
    public function verificarUsoParcela($nrparcelasnegcontr, $nrorg) {
        $params = array(
            "NRPARCELASNEGCONTR" => $nrparcelasnegcontr,
            "NRORG" => $nrorg
        );
        
        $dataset = $this->negociacaofactory->getQtdRegistrosUtilizandoParcela($params);
        return (int)$dataset[0]["QTD"] > 0;
    }
}

?>