<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00029000LocalAtendimentoFactory as LocalAtendFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class Sm00029000LocalAtendimentoService {
    
    protected $connection;
    protected $LocalAtendFactory;
    protected $environment;

    public function __construct(EntityManager $entityManager, LocalAtendFactory $LocalAtendFactory, environment $environment, NewCode $newCode) {
        $this->connection        = $entityManager->getConnection();
        $this->LocalAtendFactory = $LocalAtendFactory;
        $this->environment       = $environment;
        $this->newCode           = $newCode;
        
    }
    
   /**
     * A função "verificaTipoHierarqOrg" Verifica se existe o tipo de Hierarquia Padrão.
     */
     
     public function verificaTipoHierarqOrg($nrorg){
    
        $params = array(
            "NRORG"  => $nrorg
        );
        
        $tpHierOrg = $this->LocalAtendFactory->verificaTipoHierarqOrg($params);
        
        return $tpHierOrg;
    }
    
    
     /**
     * Insere as tabelas ParcNegocio, EndercoParc, EstruturaM, EstruturaH e HierarquiaOrg
     */
    public function inserirTudo($row) {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrg();
            
            /* Tipo de Hierarquia */
            $tpHierOrg = $this->verificaTipoHierarqOrg($nrorg);
            
            if(count($tpHierOrg) > 0){ /* Se ja tiver passa o valor para a  $tipoHierarqOrg */
                $tipoHierarqOrg = $tpHierOrg[0]["NRTIPOHIERARQORG"];
            }
            else{ /* Se não tiver cria e passa o valor para a  $tipoHierarqOrg */
                $tipoHierarqOrg = $this->inserirTipoHierarquiaOrg($row);
            }
           
            /* Parceiro de Negócio */ 
            $parcnegocio =  $this->inserirParcNegocio($row);
            
            /* Endereço do Parceiro de Negócio */
            $this->inserirEnderecoParc($row, $parcnegocio);
            
            /* Estrutura M */
            $estruturam = $this->inserirEstruturaM($row, $parcnegocio);
            
            /* Estrutura H */
            $this->inserirEstruturaH($row, $parcnegocio, $estruturam); 
            
            /* Hierarquia */
            $this->inserirHierarquiaOrg($row, $estruturam, $tipoHierarqOrg);
            
            $this->connection->commit();
            
            return $estruturam;
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Edita as tabelas ParcNegocio, EndercoParc, EstruturaM, EstruturaH e HierarquiaOrg
     */
    public function editarTudo($row) {
        $this->connection->beginTransaction();
        try {
            /* Parceiro de Negócio */ 
            $this->editarParcNegocio($row);
            
            /* Endereço do Parceiro de Negócio */
            $this->editarEnderecoParc($row);
            
            /* Estrutura M */
           $this->editarEstruturaM($row);
            
            /* Estrutura H */
            $this->editarEstruturaH($row); 
            
            /* Hierarquia */
            $this->editarHierarquiaOrg($row);
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
     /**
     * Exclui as tabelas HierarquiaOrg, EstruturaH, EstruturaM, EndercoParc e ParcNegocio
     */
    public function deletarTudo($row) {
        
        $nrparcnegocio = $row['NRPARCNEGOCIO'];
        $nrestruturam = $row['NRESTRUTURAM'];
        $nrorg = $this->environment->getNrOrg();
         
        $params = array(
            "NRESTRUTURAM" => $nrestruturam,
            "NRORG"        => $nrorg
        );
        
        $filhos = $this->LocalAtendFactory->recebefilhos($params);
        
        $this->connection->beginTransaction();

        try {
            
            foreach($filhos as $filho){
                
                $estruturam = $filho['NRESTRUTURAM'];
                $parcnegocio = $filho['NRPARCNEGOCIO'];
                
                /* Hierarquia */
                $this->deletarHierarquiaOrg($estruturam);
                
                /* Estrutura H */
                $this->deletarEstruturaH($estruturam); 
                
                /* Estrutura M */
                $this->deletarEstruturaM($estruturam);
                
                /* Endereço do Parceiro de Negócio */
                $this->deletarEnderecoParc($parcnegocio);
                
                /* Parceiro de Negócio */ 
                $this->deletarParcNegocio($parcnegocio);
            }

            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Salva o Tipo de Hierarquia.
     */
    public function inserirTipoHierarquiaOrg($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("TIPOHIERARQORG", $nrorg, 1, 10, true);
        
        $params = array(
            "NRTIPOHIERARQORG"  => $novoCodigo,
            "NMTIPOHIERARQORG"  => 'Padrão - Local de Atendimento SM',
            "NRTIPOESTRUTSUP"   => $row["NRTIPOESTRUTURA"],
            "NRTIPOESTRUTREL"   => $row["NRTIPOESTRUTURA"],
            "NRORG"             => $nrorg,
            "DTINCLUSAO"        => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperador,
            "IDATIVO"           => "S"
        );
        
        $this->LocalAtendFactory->inserirTipoHierarquiaOrg($params);
        
        return $novoCodigo;
    }
    
     /**
     * Salva o Parceiro de Negócio.
     */
    public function inserirParcNegocio($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create(); 
            
        $novoCodigo = $this->newCode->getCodeByTableName("PARCNEGOCIO", $nrorg, 1, 10, true);
        
        $params = array(
            "NRPARCNEGOCIO"       => $novoCodigo,
            "NMPRINCIPALPARC"     => $row["NMESTRUTURAM"],
            "NMSECUNDARIPARC"     => $row["NMESTRUTURAM"],
            "NRINSCRICAOPARC"     =>  $row["NRINSCRICAOPARC"],
            "CDTIPOPARCPRINCIPAL" => "ESTRUTURA",
            "CDTIPOINSCRICAO"     => $row["CDTIPOINSCRICAO"],
            "IDPESSOAFISICA"      => "N",
            "IDINSTITUICAO"       => "S",
            "IDPARCFUNDIDO"       => "N",
            "NRORG"               => $nrorg,
            "DTINCLUSAO"          => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"       => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador,
            "IDATIVO"             => "S"
        );
        
        $this->LocalAtendFactory->inserirParcNegocio($params);
        
        return $novoCodigo;
    }
    
     /**
     * atualiza o Parceiro de Negócio.
     */
    public function editarParcNegocio($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMPRINCIPALPARC"     => $row["NMESTRUTURAM"],
            "NMSECUNDARIPARC"     => $row["NMESTRUTURAM"],
            "NRINSCRICAOPARC"     => $row["NRINSCRICAOPARC"],
            "CDTIPOINSCRICAO"     => $row["CDTIPOINSCRICAO"],
            "CDOPERULTATU"        => $cdoperador,
            "NRORG"               => $nrorg,
            "NRPARCNEGOCIO"       => $row["NRPARCNEGOCIO"]
        );
        
        $this->LocalAtendFactory->editarParcNegocio($params);
    }
    
     /**
     * Deleta o Parceiro de Negócio.
     */
    public function deletarParcNegocio($nrparncnegocio) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio,
            "NRORG"               => $nrorg
        );
        
        $this->LocalAtendFactory->deletarParcNegocio($params);
    }

     /**
     * Salva a Estrutura M.
     */
    public function inserirEstruturaM($row, $parcnegocio) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
            
        $novoCodigo = $this->newCode->getCodeByTableName("ESTRUTURAM", $nrorg, 1, 10, true);
        
        $params = array(
            "NRESTRUTURAM"    => $novoCodigo,
            "NMESTRUTURAM"    => $row["NMESTRUTURAM"],
            "NRTIPOESTRUTURA" => $row["NRTIPOESTRUTURA"],
            "DTINIVIGENCIA"   => $dataAtual->format('d/m/Y'),         
            "NRPARCNEGOCIO"   => $parcnegocio,
            "CDCENTCUST"      => $row["CDCENTCUST"],
            "NRORG"           => $nrorg,
            "DTINCLUSAO"      => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"   => $nrorg,
            "CDOPERINCLUSAO"  => $cdoperador,
            "IDATIVO"         => "S"
        );
        
        $this->LocalAtendFactory->inserirEstruturaM($params);
        
        return $novoCodigo;
    }    
    
     /**
     * Atualiza a Estrutura M.
     */
    public function editarEstruturaM($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMESTRUTURAM"    => $row["NMESTRUTURAM"],
            "NRTIPOESTRUTURA" => $row["NRTIPOESTRUTURA"],
            "CDCENTCUST"      => $row["CDCENTCUST"],
            "CDOPERULTATU"    => $cdoperador,
            "NRORG"           => $nrorg,
            "NRESTRUTURAM"    => $row["NRESTRUTURAM"]
        );
        
        $this->LocalAtendFactory->editarEstruturaM($params);
    }   
    
    /**
     * Deleta a Estrutura M.
     */
    public function deletarEstruturaM($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEstruturaM($params);
    }   
    
    /**
     * Salva a Estrutura H.
     */
    public function inserirEstruturaH($row, $parcnegocio, $estruturam) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("ESTRUTURAH", $nrorg, 1, 10, true);
        
        $params = array(
            "NRESTRUTURAH"   => $novoCodigo,
            "NMESTRUTURAH"   => $row["NMESTRUTURAM"],
            "NRESTRUTURAM"   => $estruturam,
            "DTMESCOMPETENC" => $dataAtual->format('d/m/Y'),
            "NRPARCNEGOCIO"  => $parcnegocio,
            "NRORG"          => $nrorg,
            "DTINCLUSAO"     => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador,
            "IDATIVO"        => "S"
        );
        
        $this->LocalAtendFactory->inserirEstruturaH($params);
    }
    
    /**
     * Atualiza a Estrutura H.
     */
    public function editarEstruturaH($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NMESTRUTURAM"   => $row["NMESTRUTURAM"],
            "CDOPERULTATU"   => $cdoperador,
            "NRORG"          => $nrorg,
            "NRESTRUTURAH"   => $row["NRESTRUTURAH"],
            "NRESTRUTURAM"   => $row["NRESTRUTURAM"]
        );
        
        $this->LocalAtendFactory->editarEstruturaH($params);
    }
    
    /**
     * Deleta a Estrutura H.
     */
    public function deletarEstruturaH($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEstruturaH($params);
    }
    
     /**
     * Salva a Hierarquia da Estrutura.
     */
    public function inserirHierarquiaOrg($row, $estruturam, $tipoHierarqOrg) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("HIERARQUIAORG", $nrorg, 1, 10, true);
        
        $params = array(
            "NRHIERARQUIAORG"  => $novoCodigo,
            "NRTIPOHIERARQORG" => $tipoHierarqOrg,
            "NRESTRUTURAMSUP"  => $row["NRESTRUTURAMSUP"], 
            "NRESTRUTURAMREL"  => $estruturam,
            "DTINIVIGENCIA"    => $dataAtual->format('d/m/Y'),
            "NRORG"            => $nrorg,
            "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"    => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDATIVO"          => "S"
        );
        
        $this->LocalAtendFactory->inserirHierarquiaOrg($params);
    }
    
     /**
     * atualiza a Hierarquia da Estrutura.
     */
    public function editarHierarquiaOrg($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "NRESTRUTURAMSUP"  => $row["NRESTRUTURAMSUP"], 
            "NRESTRUTURAMREL"  => $row["NRESTRUTURAM"], 
            "CDOPERULTATU"     => $cdoperador,
            "NRORG"            => $nrorg,
            "NRHIERARQUIAORG"  => $row["NRHIERARQUIAORG"]
        );
        
        $this->LocalAtendFactory->editarHierarquiaOrg($params);
    }
    
     /**
     * Deleta a Hierarquia da Estrutura.
     */
    public function deletarHierarquiaOrg($nrestruturam) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam,
            "NRORG"           => $nrorg
        );
        
        $this->LocalAtendFactory->deletarHierarquiaOrg($params);
    }
    
    /**
     * Salva o endereço de Hierarquia da Estrutura.
     */
    public function inserirEnderecoParc($row, $parcnegocio) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $dataAtual = date_create();
        
        $novoCodigo = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
        
        $params = array(
            "NRENDERECOPARC"   => $novoCodigo,
            "NRPARCNEGOCIO"    => $parcnegocio, 
            "CDTIPOENDERECO"   => "PRINCIPAL",
            "DSENDERECO"       => $row["DSENDERECO"],
            "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
            "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
            "CDPAIS"           => $row["CDPAIS"],
            "SGESTADO"         => $row["SGESTADO"],
            "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
            "CDBAIRRO"         => $row["CDBAIRRO"],
            "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
            "NRCEPENDERECO"    => $row["NRCEPENDERECO"],
            "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
            "NRORG"            => $nrorg,
            "DTINCLUSAO"       => $dataAtual->format('d/m/Y'),
            "NRORGINCLUSAO"    => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDATIVO"          => "S"
        );
        
        $this->LocalAtendFactory->inserirEnderecoParc($params);
    }
    
    /**
     * atualiza o endereço de Hierarquia da Estrutura.
     */
    public function editarEnderecoParc($row) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        
        $params = array(
            "DSENDERECO"       => $row["DSENDERECO"],
            "NRIMOVELENDERECO" => $row["NRIMOVELENDERECO"],
            "DSCOMPLEENDERECO" => $row["DSCOMPLEENDERECO"],
            "CDPAIS"           => $row["CDPAIS"],
            "SGESTADO"         => $row["SGESTADO"],
            "CDMUNICIPIO"      => $row["CDMUNICIPIO"],
            "CDBAIRRO"         => $row["CDBAIRRO"],
            "NMBAIRROENDERECO" => $row["NMBAIRROENDERECO"],
            "NRCEPENDERECO"    => $row["NRCEPENDERECO"],
            "DSREFERENCIAENDE" => $row["DSREFERENCIAENDE"],
            "CDOPERULTATU"     => $cdoperador,
            "NRORG"            => $nrorg,
            "NRENDERECOPARC"   => $row["NRENDERECOPARC"],
            "NRPARCNEGOCIO"    => $row["NRPARCNEGOCIO"]
        );
        
        $this->LocalAtendFactory->editarEnderecoParc($params);
    }
    
     /**
     * Deleta o endereço de Hierarquia da Estrutura.
     */
    public function deletarEnderecoParc($nrparncnegocio) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio,
            "NRORG"               => $nrorg
        );
        
        $this->LocalAtendFactory->deletarEnderecoParc($params);
    }
    
}

?>
